# Author: Jen Boulware
# Date: October 2011
# Purpose: Common scripts for CSC Toolbox
# Requirements: Spatial Analyst Extension



# Import arcpy, sys and traceback modules
import sys, traceback
import datetime
import os
import arcpy
from arcpy import env
from arcpy.sa import *
from xml.dom.minidom import parse


class NotTextNodeError:
    pass


logfilePath = 'C:/arcgis/cscToolboxLog3.txt'
logfile = open(logfilePath,  'a+') 

def getLogFile():  
    # Create or replace log file
    global logfile
    logfile = open(logfilePath,  'a+')
    now = datetime.datetime.now() 
    print 'start time:', now.ctime()
    logfile.write("Started Process"  + now.ctime() + "\n") 

def closeLogFile():  
    global logfile
    if (logfile):
        now = datetime.datetime.now() 
        print 'end time:', now.ctime()
        logfile.write("Ended Process"  + now.ctime() + "\n") 
        logfile.close()

def AddPrintMessage(msg, severity):
    global logfile
    print msg
    logfile.write(msg + "\n")
    if severity == 0: arcpy.AddMessage(msg)
    elif severity == 1: arcpy.AddWarning(msg)
    elif severity == 2: arcpy.AddError(msg)


def checkSpatialLic(): 
    # Check out the ArcGIS Spatial Analyst extension license 
    if arcpy.CheckExtension("Spatial") == "Available":
        arcpy.CheckOutExtension("Spatial")
        return 0
    else:
        return 1


def checkPassedParamCount(cnt): 
    # Since this is a common check, we pass the expected number
    #simple getArgumentCount tells us if they don't have enough
    #returns false if not, which raises the error in the main code
    if arcpy.GetArgumentCount() < cnt:
        return 1
    else:
        return 0


def getTextFromNode(node):
    """
    scans through all children of node and gathers the
    text. if node has non-text child-nodes, then
    NotTextNodeError is raised.
    """
    t = ""
    emptyNode = node.hasChildNodes()
    if emptyNode:
        for n in node.childNodes:
            if n.nodeType == n.TEXT_NODE:
                t += n.nodeValue
            else:
                raise NotTextNodeError
    else:
        t = "Null"
    return t

def nodeToDic(node):

    dic = {}
    multlist = {} # holds temporary lists where there are multiple children
    multiple = 0
    for n in node.childNodes:
        if n.nodeType != n.ELEMENT_NODE:
            continue

        # find out if there are multiple records
        if len(node.getElementsByTagName(n.nodeName)) > 1:
            multiple = 1
            # and set up the list to hold the values
            if not multlist.has_key(n.nodeName):
                multlist[n.nodeName] = []

        try:
            #text node
            text = getTextFromNode(n)
        except NotTextNodeError:
            if multiple:
                # append to our list
                multlist[n.nodeName].append(nodeToDic(n))
                dic.update({n.nodeName:multlist[n.nodeName]})
                continue
            else:
                # 'normal' node
                dic.update({n.nodeName:nodeToDic(n)})
                continue

        # text node
        if multiple:
            multlist[n.nodeName].append(text)
            dic.update({n.nodeName:multlist[n.nodeName]})
        else:
            dic.update({n.nodeName:text})
    return dic

def readXML(filename):
    dom = parse(filename)
    return nodeToDic(dom)

